from re import *
from node import *
from linkedlist import *

SCM_TOKENS     = ['\(', '\)', "'\(", '"', '\,', '\`', ';', ' +', '\n']
REG_SCM_TOKENS = '|'.join(SCM_TOKENS)
OPEN           = '('
CLOSE          = ')'
SINGLE_QUOTE   = "'("
DOUBLE_QUOTE   = '"'
COMMENT        = ';'
NEW_LINE       = '\n'

def scan(source):
    """ 
    This function returns a AST.

    It receives a function that returns a stream of characterers 
    whenever it is called. 

    The stream of characters is scanned and when it is a complete
    scheme statement it returns the AST for the statement. 
    """

    seq   = finditer(REG_SCM_TOKENS, source)
    prev  = 0

    for ind in seq:
        start = ind.start()
        end   = ind.end()
        id    = source[prev:start]

        yield (id, prev, start)

        id    = ind.group()

        yield (id, start, end)

        prev  = end

def readstr(sequence):
    """ 
        It consumes a sequence to build a string.
        When it finds a " it stsexps and returns a
        String node. 
    """

    data = []
    for id, start, end in sequence:
        if id == DOUBLE_QUOTE:
            break
        else:
            data.append(id)
    return Str(''.join(data))

  
def until(sequence, cond):
    for id, start, end in sequence:
        if cond(id):
            break

def acc(sequence, args=[]):
    for id, start, end in sequence:
        if id == OPEN:
            node = acc(sequence, [])
            args.append(node)
        elif id == CLOSE:
            return Sexp(*args)
        elif id == SINGLE_QUOTE:
            quote = Quote(acc(sequence, []))
            args.append(quote)
        elif id == DOUBLE_QUOTE:
            node = readstr(sequence)
            args.append(node)
        elif id.isspace():
            pass
        elif id == COMMENT:
            until(sequence, lambda id: id == NEW_LINE)
        elif id:
            atom = readtype(id)
            args.append(atom)

    return Main(*args)

















